'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Emoji;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _emoji = require('../data/emoji');

var _emoji2 = _interopRequireDefault(_emoji);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LARGE_MULTIPLIER = 3; /**
                           * @copyright   2016, Miles Johnson
                           * @license     https://opensource.org/licenses/MIT
                           * 
                           */

function Emoji(_ref) {
  var shortname = _ref.shortname,
      unicode = _ref.unicode,
      emojiPath = _ref.emojiPath,
      emojiSize = _ref.emojiSize,
      _ref$enlargeEmoji = _ref.enlargeEmoji,
      enlargeEmoji = _ref$enlargeEmoji === undefined ? false : _ref$enlargeEmoji;

  if (!shortname && !unicode) {
    throw new Error('Emoji component requires a `unicode` character or a `shortname`.');
  }

  if (unicode && !_emoji.UNICODE_TO_SHORTNAME[unicode] || shortname && !_emoji.SHORTNAME_TO_UNICODE[shortname]) {
    return _react2.default.createElement(
      'span',
      null,
      unicode || shortname
    );
  }

  if (!shortname && unicode) {
    shortname = _emoji.UNICODE_TO_SHORTNAME[unicode];
  } else if (!unicode && shortname) {
    unicode = _emoji.SHORTNAME_TO_UNICODE[shortname];
  }

  var emoji = _emoji2.default[shortname];
  var className = ['interweave__emoji'];
  var styles = {};

  if (emojiSize) {
    styles.display = 'inline-block';
    styles.verticalAlign = 'middle';
    styles.width = emojiSize + 'em';
  }

  if (enlargeEmoji) {
    className.push('interweave__emoji--large');

    if (emojiSize) {
      styles.width = emojiSize * LARGE_MULTIPLIER + 'em';
    }
  }

  var path = emojiPath || '{{hexcode}}';

  if (typeof path === 'function') {
    path = path(emoji.hexcode, enlargeEmoji, emojiSize);
  } else {
    path = path.replace('{{hexcode}}', emoji.hexcode);
  }

  return _react2.default.createElement('img', {
    src: path,
    alt: unicode,
    style: styles,
    className: className.join(' '),
    'data-unicode': unicode,
    'data-hexcode': emoji.hexcode,
    'data-codepoint': emoji.codepoint.join('-'),
    'data-shortname': shortname
  });
}

Emoji.propTypes = {
  emojiPath: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
  emojiSize: _propTypes2.default.number,
  enlargeEmoji: _propTypes2.default.bool,
  shortname: _propTypes2.default.string,
  unicode: _propTypes2.default.string
};

Emoji.defaultProps = {
  shortname: '',
  unicode: '',
  emojiPath: '{{hexcode}}',
  emojiSize: 0,
  enlargeEmoji: false
};